"""
Data structures for sparse float data. Life is made simpler by dealing only
with float64 data
"""

# pylint: disable=W0611
# flake8: noqa

from pandas.sparse.series import SparseSeries
from pandas.sparse.frame import SparseDataFrame
from pandas.sparse.panel import SparsePanel
